% STUDY = pop_std_envtopo() - called by eegplugin_std_envtopo. Runs std_envtopo.
%
% Usage:
%   >>  STUDY = pop_std_envtopo(STUDY, ALLEEG, 'key', val);
%
% Inputs:
%   STUDY   - an EEGLAB STUDY structure containing EEG structures
%   ALLEEG  - the ALLEEG data structure; can also be an EEG dataset structure.
%
% Optional inputs:
%   same as std_envtopo
%
% Outputs:
%   STUDY  - an EEGLAB STUDY structure containing EEG structures
%
% Author: Makoto Miyakoshi, Arnaud Delorme JSPS/SCCN, INC, UCSD
%
% See also: eegplugin_std_envtopo, pop_envtopo, std_envtopo, envtopo

% Copyright (C) 2011, Makoto Miyakoshi JSPS/SCCN,INC,UCSD.
%
% History
% 10/08/2013 ver 2.4 by Makoto. Displays tic toc results with sprintf. std_envtopo 1.0 compatibility.
% 10/02/2013 ver 2.3 by Makoto. envtopoParams{1,9} now contains convTopoErpAllClusters. Brushed up erpdata loading process.
% 08/15/2013 ver 2.2 by Makoto. Help implemented. 
% 05/09/2013 ver 2.1 by Makoto. Store user input.
% 01/30/2013 ver 2.0 by Makoto. STUDY = std_envtopo()
% 04/25/2012 ver 1.9 by Makoto. Description of how to apply low-pass filter added. History function disabled being unable to solve a bug.
% 01/24/2012 ver 1.8 by Makoto. Anti-crash for the cancel bottun. eegh support being attempted
% 01/17/2012 ver 1.7 by Makoto. Input box for 'diff' added.
% 01/13/2012 ver 1.6 by Makoto. Limitation for reduced groups added.
% 01/12/2012 ver 1.5 by Makoto. Limitation is changed to CAUTION.
% 12/26/2011 ver 1.4 by Makoto. Limitation notices added.
% 10/11/2011 ver 1.3 by Makoto. Line75- fixed.
% 10/10/2011 ver 1.2 by Makoto. Two types of scalp topos can be presented.
% 10/06/2011 ver 1.1 by Makoto. Line 77 'design', STUDY.currentdesign.
% 08/01/2011 ver 1.0 by Makoto. Created.

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function STUDY = pop_std_envtopo(STUDY, ALLEEG, varargin)

if nargin < 2
    help pop_std_envtopo;
    return;
end;

try
    userInput = inputgui('title', 'pop_std_envtopo()', 'geom', ...
           {{2 12 [0 0] [1 1]} {2 12 [1 0] [1 1]} ...
            {2 12 [0 1] [1 1]} {2 12 [1 1] [1 1]} ...
            {2 12 [0 2] [1 1]} {2 12 [1 2] [1 1]} ...
            {2 12 [0 3] [1 1]} {2 12 [1 3] [1 1]} ...
            {2 12 [0 4] [1 1]} {2 12 [1 4] [1 1]} ...
            {2 12 [0 5] [1 1]} {2 12 [1 5] [1 1]} ...
            {1 12 [0 6] [1 1]}...
            {2 12 [0 7] [1 1]} {2 12 [1 7] [1 1]} ...
            {2 12 [0 8] [1 1]} {2 12 [1 8] [1 1]} ...
            {1 12 [0 10] [1 1]}...
            {1 12 [0 11] [1 1]}...
            {1 12 [0 12] [1 1]}...
            {6 12 [0 13] [1 1]}...
           }, 'uilist', ...
           {{ 'style' 'text' 'string' 'Time range (in ms) to plot (Ex: -200 500, not -200:500):' } { 'style' 'edit' 'string' STUDY.envtopoParams{1,1} } ...
            { 'style' 'text' 'string' 'Time range (in ms) to rank cluster contributions:' } { 'style' 'edit' 'string' STUDY.envtopoParams{1,2} } ...
            { 'style' 'text' 'string' 'Number of largest contributing clusters to plot (Default: 7)' } { 'style' 'edit' 'string' STUDY.envtopoParams{1,3} } ...
            { 'style' 'text' 'string' 'Else plot these cluster numbers only (Ex: 2:4,7):' } { 'style' 'edit' 'string' STUDY.envtopoParams{1,4}} ...
            { 'style' 'text' 'string' 'Cluster numbers to exclude: ' } { 'style' 'edit' 'string' STUDY.envtopoParams{1,5}} ...
            { 'style' 'text' 'string' 'Subtraction. 2 1 1 1 = Cond2 Group1 minus Cond1 Group1.'} { 'style' 'edit' 'string' STUDY.envtopoParams{1,6}}...
            { 'style' 'text' 'string' 'For interaction, 2 2 1 2 2 1 1 1 = (C2G2-C1G2) - (C2G1-C1G1).' }...
            { 'style' 'text' 'string' 'Include only the clusters that were part of the clustering' } { 'style' 'checkbox', 'value', STUDY.envtopoParams{1,7}} ...
            { 'style' 'text' 'string' 'Optional inputs (see Help)' } { 'style' 'edit' 'string' STUDY.envtopoParams{1,8}} ...
            { 'style' 'text' 'string' '++++++++++++++++   Run STUDY = rmfield(STUDY, ''envtopoParams''); if you modify clusters.'}...
            { 'style' 'text' 'string' '++++ CAUTION!!! ++++    Group MUST be the second variable in the STUDY design. All the clusters MUST contain ICs.' }...
            { 'style' 'text' 'string' '++++++++++++++++   DO NOT change the name ''outlier 2''. Plot ERPs ''Params'' time range option will be reset.'}...
            { 'style', 'pushbutton', 'string', 'Help', 'callback', 'pophelp(''std_envtopo'');'}});
catch
    userInput = inputgui('title', 'Plot envtopo', 'geom', ...
           {{2 12 [0 0] [1 1]} {2 12 [1 0] [1 1]} ...
            {2 12 [0 1] [1 1]} {2 12 [1 1] [1 1]} ...
            {2 12 [0 2] [1 1]} {2 12 [1 2] [1 1]} ...
            {2 12 [0 3] [1 1]} {2 12 [1 3] [1 1]} ...
            {2 12 [0 4] [1 1]} {2 12 [1 4] [1 1]} ...
            {2 12 [0 5] [1 1]} {2 12 [1 5] [1 1]} ...
            {1 12 [0 6] [1 1]}...
            {2 12 [0 7] [1 1]} {2 12 [1 7] [1 1]} ...
            {2 12 [0 8] [1 1]} {2 12 [1 8] [1 1]} ...
            {1 12 [0 10] [1 1]}...
            {1 12 [0 11] [1 1]}...
            {1 12 [0 12] [1 1]}...
            {6 12 [0 13] [1 1]}...
           }, 'uilist', ...
           {{ 'style' 'text' 'string' 'Time range (in ms) to plot (Ex: -200 500, not -200:500):' } { 'style' 'edit' 'string' '' } ...
            { 'style' 'text' 'string' 'Time range (in ms) to rank cluster contributions:' } { 'style' 'edit' 'string' '' } ...
            { 'style' 'text' 'string' 'Number of largest contributing clusters to plot (Default: 7)' } { 'style' 'edit' 'string' '7' } ...
            { 'style' 'text' 'string' 'Else plot these cluster numbers only (Ex: 2:4,7):' } { 'style' 'edit' 'string' '' } ...
            { 'style' 'text' 'string' 'Cluster numbers to exclude:' } { 'style' 'edit' 'string' '' } ...
            { 'style' 'text' 'string' 'Subtraction. Ex. 1 1 2 1 = Cond1 Group1 minus Cond2 Group1.'} { 'style' 'edit' 'string' '' }...
            { 'style' 'text' 'string' 'For interaction, 2 2 1 2 2 1 1 1 = (C2G2-C1G2) - (C2G1-C1G1).' }...
            { 'style' 'text' 'string' 'Include only the clusters that were part of the clustering' } { 'style' 'checkbox', 'value', 1} ...
            { 'style' 'text' 'string' 'Optional inputs (see Help)' } { 'style' 'edit' 'string' '' } ...
            { 'style' 'text' 'string' '++++++++++++++++   Run STUDY = rmfield(STUDY, ''envtopoParams''); if you modify clusters.'}...
            { 'style' 'text' 'string' '++++ CAUTION!!! ++++    Group MUST be the second variable in the STUDY design. All the clusters MUST contain ICs.' }...
            { 'style' 'text' 'string' '++++++++++++++++   DO NOT change the name ''outlier 2''. Plot ERPs ''Params'' time range option will be reset.'}...
            { 'style', 'pushbutton', 'string', 'Help', 'callback', 'pophelp(''std_envtopo'');'}});
end

% if canceled, escape
if isempty(userInput), return, end

% prepare/update STUDY.envtopoParams 
if isfield(STUDY, 'envtopoParams') && length(STUDY.envtopoParams)==9
    for n = 1:8
        STUDY.envtopoParams{1,n} = userInput{1,n};
    end
else
    STUDY.envtopoParams = [userInput {[]}];
end

% prepare optional inputs for std_envtopo
options = '';
if ~isempty( userInput{1} ), options = [ options '''timerange'',[' userInput{1} '],' ]; end;
if ~isempty( userInput{2} ), options = [ options '''limitcontribtime'',[' userInput{2} '],' ]; end;
if ~isempty( userInput{3} ), options = [ options '''clustnums'',[-' userInput{3} '],' ]; end;
if ~isempty( userInput{4} ), options = [ options '''clustnums'',[' userInput{4} '],' ]; end;
if ~isempty( userInput{5} ), options = [ options '''clust_exclude'',[' userInput{5} '],' ]; end;
if ~isempty( userInput{6} ), options = [ options '''diff'',[' userInput{6} '],' ]; end;
if userInput{7} == 1; options = [ options '''onlyclus'', ''on''']; else options = [ options '''onlyclus'', ''off''']; end
if ~isempty( userInput{8} ), options = [ options ',' userInput{8} ]; end;
arguments = eval([ '{' options '}' ]);

if length(STUDY.cluster) == 1
    errordlg2('Cannot plot envtopo with the parent cluster only');
end;

if ~isfield(STUDY.cluster, 'erpdata') % no erpdata present under STUDY.cluster
    disp('Loading cluster ERPs... (done only once)')
    STUDY = std_readerp(STUDY, ALLEEG, 'design', STUDY.currentdesign, 'clusters', 2:length(STUDY.cluster), 'singletrials', 'off', 'timerange', [ALLEEG(1,1).xmin*1000 ALLEEG(1,1).xmax*1000]);
elseif ~isempty(STUDY.etc.erpparams.timerange) || length(STUDY.cluster(1,2).erptimes) ~= length(ALLEEG(1,1).times) % if erpdata under STUDY.cluster is wrong
    STUDY.etc.erpparams.timerange = [];
    disp('Warning: erpparams time range is set to none.')
    disp('Loading cluster ERPs... (done only once)')
    STUDY = std_readerp(STUDY, ALLEEG, 'design', STUDY.currentdesign, 'clusters', 2:length(STUDY.cluster), 'singletrials', 'off', 'timerange', [ALLEEG(1,1).xmin*1000 ALLEEG(1,1).xmax*1000]);
else
    disp('No need to load erpdata.')
end

% update STUDY in the base workspace
assignin('base', 'STUDY', STUDY);

% run std_evntopo
tic;
STUDY = std_envtopo(STUDY, ALLEEG, arguments{:});
measuredTime = toc;
resultReport = sprintf('std_envtopo computation time: %3.0f seconds.',round(measuredTime));
disp(resultReport)